using GrapeCity.Documents.Excel;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Globalization;
using System.Runtime.InteropServices;
using System.Linq;

namespace CustomFunctionTest
{
    class Program
    {
        static void Main(string[] args)
        {
            Workbook.AddCustomFunction(new HighestValues());
            var workbook = new Workbook();
            workbook.Open("SimpleBudget.xlsx");
            workbook.Worksheets[0].Range["B25"].Formula = "HighestValues(B11:C23, 1, 2)";
            var result = workbook.Worksheets[0].Range["B25"].Value;

            int rowIndex, columnIndex;
            GrapeCity.Documents.Excel.CellInfo.CellNameToIndex("C25", out rowIndex, out columnIndex);
            workbook.Worksheets[0].Range[rowIndex, columnIndex].Value = result;

            workbook.Save("SimpleBudget.xlsx");
           
        }
    }
}
public class HighestValues : CustomFunction
{
    public HighestValues()
        : base("HighestValues", FunctionValueType.Text, new Parameter[] { new Parameter(FunctionValueType.Object), new Parameter(FunctionValueType.Number), new Parameter(FunctionValueType.Number) })
    {

    }

    public override object Evaluate(object[] arguments, ICalcContext context)
    {
        if (arguments.Length < 3)
        {
            return CalcError.Value;
        }

        var values = arguments[0];
        if (values is IEnumerable<object>)
        {
            values = (values as IEnumerable<object>).FirstOrDefault();
        }
        object[,] array = values as object[,];
        if (array == null)
        {
            return CalcError.Value;
        }
        int rowCount = array.GetLength(0);
        int colCount = array.GetLength(1);
        if (rowCount <= 0 || colCount <= 0)
        {
            return CalcError.Value;
        }
        int resultCol = (int)(double)arguments[1] - 1;
        if (resultCol < 0 || resultCol >= colCount)
        {
            return CalcError.Num;
        }

        int numberCol = (int)(double)arguments[2] - 1;
        if (numberCol < 0 || resultCol >= colCount)
        {
            return CalcError.Num;
        }

        List<temp> list = new List<temp>();
        for (int i = 0; i < rowCount; i++)
        {
            string text = array[i, resultCol]?.ToString();
            double number = array[i, numberCol] is double ? (double)array[i, numberCol] : 0;
            list.Add(new temp(text, number));
        }
        list.Sort((x, y) =>
        {
            if (x.Number > y.Number)
            {
                return -1;
            }
            else if (x.Number == y.Number)
            {
                return 0;
            }
            else
            {
                return 1;
            }
        });
        string result = null;
        int count = Math.Min(list.Count, 2);
        for (int i = 0; i < count; i++)
        {
            if (result != null)
            {
                result += ",";
            }
            result += list[i].Text;
        }

        return result;
    }

    private class temp
    {
        public string Text;
        public double Number;

        public temp(string text, double number)
        {
            this.Text = text;
            this.Number = number;
        }
    }
}
